#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sbus.h"

int speed, lr;

void
setGroin( int bRight, int value )
{
  int chGroin = bRight ? 17 : 11;
  int chKnee = bRight ? 18 : 12;
  int chAnkle = bRight ? 19 : 13;
  int knee = sbus_getnext( chKnee ) / PWM_DEG;
  sbus_setnext( chGroin, value * PWM_DEG );
  sbus_setnext( chAnkle, (knee - value + 3) * PWM_DEG );
}

void
setKnee( int bRight, int value )
{
  int chGroin = bRight ? 17 : 11;
  int chKnee = bRight ? 18 : 12;
  int chAnkle = bRight ? 19 : 13;
  int groin = sbus_getnext( chGroin ) / PWM_DEG;
  sbus_setnext( chKnee, value * PWM_DEG );
  sbus_setnext( chAnkle, (value - groin + 3) * PWM_DEG );
}

void
setHip( int value )
{
  sbus_setnext( 10,   value * PWM_DEG );
  sbus_setnext( 16, - value * PWM_DEG );
  sbus_setnext( 20, - value * PWM_DEG );
  sbus_setnext( 14,   value * PWM_DEG );
}

int
home( int value )
{
  setGroin( 0, value );
  setKnee( 0, value * 2 );
  setGroin( 1, value );
  setKnee( 1, value * 2 );
  sbus_setnext( 10, 0 * PWM_DEG );
  sbus_setnext( 16, 0 * PWM_DEG );
  sbus_setnext( 20, 0 * PWM_DEG );
  sbus_setnext( 14, 0 * PWM_DEG );
}

int
main( int argc, char *argv[] )
{
  int value, i, bRight;

  if ( argc < 2 ) {
    printf( "%s angle [speed]\n", argv[0] );
    exit(1);
  }
  value = atoi( argv[1] );
  if ( value < 0 ) {
    printf( "%s: invalud angle\n", argv[0] );
    exit(1);
  }
  speed = argc > 2 ? atoi( argv[2] ) : 50;
  printf("SBUS INIT :");
  if ( sbus_init() != 0 ) {
    printf("NG\n");
    exit(1);
  }
  printf("OK\n");
  sbus_reset();
  sbus_usetrim(1);
  sbus_getwidth();
  sbus_setpwm();

  sbus_initnext();
  home( value );
  sbus_printnext();
  sbus_movesync( speed );

  for ( i = 0, bRight = 0; i < 4; i ++, bRight = 1 - bRight ) {
    sbus_initnext();
    setHip( bRight ? -20 : 20 );
    sbus_movesync( speed / 2 );

    sbus_initnext();
    setHip( bRight ? -17 : 17 );
    setGroin( bRight, value + 20 );
    setKnee( bRight, (value + 20) * 2 );
    sbus_movesync( speed );

    //sleep( 1 );

    sbus_initnext();
    setGroin( bRight, value + 10 );
    setKnee( bRight, (value + 10) * 2 );
    sbus_movesync( speed );

    sbus_initnext();
    home( value );
    sbus_movesync( speed / 2 );

    sleep ( 1 );
  }
  return 0;
}
