/*
 * rmeye.c
 * Originally Created: iXs Research Co.,Ltd.
 * Modified: 2010/01/20, Eiichiro Ito(GHC02331@nifty.com)
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <time.h>
#include <sys/types.h>
#include "rmeye.h"

#define AM1_PORT     "/dev/ttyAM1"
#define BAUD_RATE    B115200
#define BUFF_SIZE    4096

#define DEBUG

#ifdef DEBUG
#define PRINTF(x) printf(x)
#define PRINTF2(x,xx) printf(x,xx)
#else
#define PRINTF(x)
#define PRINTF2(x,xx)
#endif

static int am1_fd = -1;
static unsigned char buffer[BUFF_SIZE];

int
rmeye_init()
{
  struct termios tio;

  if ( am1_fd < 0 ) {
    printf("rmeye_init\n");
    am1_fd = open(AM1_PORT, O_RDWR|O_NONBLOCK);
    if (am1_fd < 0) {
      PRINTF("RM-eye OpenError\n");
      return -1;
    }
    memset(&tio, 0, sizeof(tio));
    tio.c_cflag = CS8 | CLOCAL | CREAD;
    tio.c_cc[VTIME] = 100;
    cfsetispeed(&tio, BAUD_RATE);
    cfsetospeed(&tio, BAUD_RATE);
    tcsetattr(am1_fd, TCSANOW, &tio);
  }
  return 0;
}

int
rmeye_getpoint( int mode, int *x, int *y, int *size )
{
  int len;
  int time_out = 20000;

  if (am1_fd < 0) {
    PRINTF("RM-eye OpenError\n");
    return -1;
  }
  buffer[0] = 0xfb;
  buffer[1] = mode;
  write(am1_fd, buffer, 2);
  while (time_out --) {
    len = read(am1_fd, buffer, BUFF_SIZE);
    if (len > 2) {
      break;
    }
  }
  if (len >= 8) {
    if (buffer[0] == 0xf0) {
      *x = ((int)buffer[2] << 8) + ((int)buffer[3]);
      *y = ((int)buffer[4] << 8) + ((int)buffer[5]);
      *size = ((int)buffer[6] << 8) + ((int)buffer[7]);
      //printf("len = %d, x = %d, y = %d, size = %d\n",
      //len, *x, *y, *size);
    } else if (buffer[0] == 0x0f) {
      PRINTF("Get 0x0F Error\n");
      return -1;
    } else {
      PRINTF2("Get serial ERROR %x \n", buffer[0]);
      return -1;
    }
  } else {
    PRINTF("GET DATA ERROR\n");
    return -1;
  }
  return 0;
}

int
rmeye_setcolor( int mode, unsigned long center, unsigned long range )
{
  int len;
  int time_out = 20000;
  int rcen = (center & 0xFF0000) >> 16;
  int gcen = (center & 0x00FF00) >> 8;
  int bcen = (center & 0x0000FF);
  int rran = (range & 0xFF0000) >> 16;
  int gran = (range & 0x00FF00) >> 8;
  int bran = (range & 0x0000FF);
  int rmax = rcen + rran > 255 ? 255 : rcen + rran;
  int gmax = gcen + gran > 255 ? 255 : gcen + gran;
  int bmax = bcen + bran > 255 ? 255 : bcen + bran;
  int rmin = rcen - rran < 0 ? 0 : rcen - rran;
  int gmin = gcen - gran < 0 ? 0 : gcen - gran;
  int bmin = bcen - bran < 0 ? 0 : bcen - bran;

  printf( "rmeye_setcolor: (%d,%d,%d)-(%d,%d,%d)\n",
	  rmax, gmax, bmax, rmin, gmin, bmin );
  if (am1_fd < 0) {
    PRINTF("RM-eye OpenError\n");
    return -1;
  }
  buffer[0] = 0xFD;
  buffer[1] = mode;
  buffer[2] = (unsigned char) rmax;
  buffer[3] = (unsigned char) gmax;
  buffer[4] = (unsigned char) bmax;
  buffer[5] = (unsigned char) rmin;
  buffer[6] = (unsigned char) gmin;
  buffer[7] = (unsigned char) bmin;
  write(am1_fd, buffer, 8);
  while (time_out --) {
    len = read(am1_fd, buffer, BUFF_SIZE);
    if (len >= 2) {
      break;
    }
  }
  printf("len=%d\n", len);
  if (buffer[0] == 0xf0) {
  } else {
    PRINTF("rmeye_setcolor: error\n");
    return -1;
  }
  return 0;
}

int
rmeye_getcolor( int mode, unsigned long *center, unsigned long *range )
{
  int len;
  int time_out = 20000;
  int rmax, gmax, bmax, rmin, gmin, bmin;
  int rcen, gcen, bcen, rran, gran, bran;

  if (am1_fd < 0) {
    PRINTF("RM-eye OpenError\n");
    return -1;
  }
  buffer[0] = 0xFC;
  buffer[1] = mode;
  write(am1_fd, buffer, 2);
  while (time_out --) {
    len = read(am1_fd, buffer, BUFF_SIZE);
    if (len >= 2) {
      break;
    }
  }
  printf("len=%d\n", len);
  if ( len >= 8 ) {
    if (buffer[0] == 0xf0) {
      rmax = (int) buffer[2];
      gmax = (int) buffer[3];
      bmax = (int) buffer[4];
      rmin = (int) buffer[5];
      gmin = (int) buffer[6];
      bmin = (int) buffer[7];
      printf( "rmeye_getcolor: (%d,%d,%d)-(%d,%d,%d)\n",
	  rmax, gmax, gmax, rmin, gmin, bmin );
      rcen = (rmax + rmin) / 2;
      gcen = (gmax + gmin) / 2;
      bcen = (bmax + bmin) / 2;
      rran = (rmax - rmin) / 2;
      gran = (gmax - gmin) / 2;
      bran = (bmax - bmin) / 2;
      *center = ((int)rcen<<16) + ((int)gcen<<8) + (int)bcen;
      *range = ((int)rran<<16) + ((int)gran<<8) + (int)bran;
    } else {
      PRINTF("rmeye_getcolor: error\n");
      return -1;
    }
  } else {
    PRINTF("rmeye_getcolor: error\n");
    return -1;
  }
  return 0;
}
