/*
 * sbus.c
 * Originally Created: iXs Research Co.,Ltd.
 * Modified: 2010/01/14, Eiichiro Ito(GHC02331@nifty.com)
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <termios.h>
#include <time.h>
#include <sys/types.h>
#include "sbus.h"

#define SBUS_PORT	"/dev/sbus"
#define SBUS_BAUD_RATE	B115200
#define BUFF_SIZE	4096
#define PWM_MIN		13640
#define PWM_MAX		41656
#define PWM_CENT	((PWM_MIN+PWM_MAX)/2)	//27648

#define DEBUG
//#define DEBUG_SBUS

#ifdef DEBUG
#define PRINTF(x) printf(x)
#define PRINTF2(x,xx) printf(x,xx)
#else
#define PRINTF(x)
#define PRINTF2(x,xx)
#endif

struct sbus_sv {
  unsigned short s[24];
} serv_curr, serv_next;
static int sbus_fd = -1;
static unsigned char buffer[BUFF_SIZE];
static int serv_dir[24] = {
  1, /* 0 */
  -1, 1,  1,  0, 1, -1, -1,  0, /* 1..8 */
  1,  1,  1, -1, -1,  1,	 /* 9..14 */
  1, -1, -1,  1,  1, -1,	 /* 15..20 */
  0,  0,  0
};
static int serv_trim[24] = {
  -620,
  -10540, -13950, 0, 0, 9455, 13485, 0, 0,
  -1085,  575, -930,  6510, -3255, 1550,
   1550, -1705,    0, -6510,  3255, 310,
  0, 0, 0
};

static int use_trim = 0;

int
sbus_init()
{
  struct termios tio;
  int i;

  for ( i = 0; i < 24; i ++ ) {
    serv_curr.s[i] = PWM_CENT;
    serv_next.s[i] = PWM_CENT;
  }
#ifndef DEBUG_SBUS
  if ( sbus_fd < 0 ) {
    printf("sbus_init\n");
    sbus_fd = open(SBUS_PORT, O_RDWR);
    if ( sbus_fd < 0 ) {
      PRINTF("SBUS OpenError\n") ;
      return -1;
    }
    memset(&tio, 0, sizeof(tio));
    tio.c_cflag = CS8 | CLOCAL | CREAD;
    tio.c_cc[VTIME] = 100;
    cfsetospeed(&tio, SBUS_BAUD_RATE);
    cfsetispeed(&tio, SBUS_BAUD_RATE);
    tcsetattr( sbus_fd, TCSANOW, &tio );
  }
#endif
  return sbus_reset();
}

int
sbus_reset()
{
#ifdef DEBUG_SBUS
  printf("sbus_reset\n");
#else
  int len = -1;
  int time_out = SBUS_TIMEOUT1;

  if ( sbus_fd < 0 ) {
    PRINTF("SBUS OpenError\n");
    return -1;
  }
  buffer[0] = 0x00;
  buffer[1] = 0x10;
  write( sbus_fd, buffer, 2 );
  PRINTF("RESET BOARD : ");
  while ( time_out -- ) {
    len = read( sbus_fd, buffer, 1);
    if ( len > 0 ) {
      break;
    }
  }
  if ( len == 1 ) {
    PRINTF("ON\n");
  } else if ( len < 0 ) {
    PRINTF("NG:DEVICE CONNECT ERROR)\n");
    return -1;
  } else {
    PRINTF("NG:UNKNOWN ERROR\n");
    return -1;
  }
#endif
  return 0;
}

int
sbus_setpwm()
{
#ifdef DEBUG_SBUS
  printf("sbus_setpwm\n");
#else
  int len = -1;
  int time_out = SBUS_TIMEOUT1;

  if ( sbus_fd < 0 ) {
    PRINTF("SBUS OpenError\n");
    return -1;
  }
  buffer[0] = 0x00;
  buffer[1] = 0x20;
  buffer[2] = 0x00;
  write( sbus_fd, buffer, 3 );
  while ( time_out -- ) {
    len = read( sbus_fd, buffer, 1 );
    if ( len > 0 ) {
      break;
    }
  }
  if ( buffer[0] == SBUS_ACK ) {
  } else {
    PRINTF("START PWM : NG");
    return -1;
  }
#endif
  return 0;
}

int
sbus_getwidth()
{
#ifdef DEBUG_SBUS
  printf("sbus_getwidth\n");
#else
  int len = -1;
  int i, j;
  int time_out = 20000;

  if ( sbus_fd < 0 ) {
    PRINTF("SBUS OpenError\n");
    return -1;
  }
  buffer[0] = 0x10;
  buffer[1] = 0x01;
  write( sbus_fd, buffer, 2 );
  while ( time_out -- ) {
    len = read( sbus_fd, buffer, BUFF_SIZE );
    if ( len > 2 ) {
      break;
    }
  }
  if ( buffer[0] == SBUS_ACK ) {
  } else {
    PRINTF("GET PWM VALUE : NG\n");
    return -1;
  }
  PRINTF2( "len = %d\n",len );
  for ( i = 1, j = 0; i < 48; i += 2, j ++ ) {
    unsigned short value = ((unsigned short)((int)buffer[i] << 8 | (int)buffer[i + 1]));
    value -= use_trim * serv_trim[j];
    serv_curr.s[j] = value;
    //PRINTF2("%d,", value - PWM_CENT);
  }
  //PRINTF("\n");
#endif
  return 0;
}

int
sbus_setwidth()
{
#ifdef DEBUG_SBUS
  printf("sbus_setwidth\n");
#else
  int len = -1;
  int i, j, csum;
  int time_out = SBUS_TIMEOUT1;

  if ( sbus_fd < 0 ) {
    PRINTF("SBUS OpenError\n");
    return -1;
  }
  buffer[0] = 0x10;
  buffer[1] = 0x00;
  for ( i = 0, j = 2; i < 24 ; i ++, j += 2 ) {	
    unsigned short value = serv_curr.s[i];
    value += use_trim * serv_trim[i];
    value = value > PWM_MAX ? PWM_MAX : value;
    value = value < PWM_MIN ? PWM_MIN : value;
    buffer[j]   = (unsigned char) (value >> 8);
    buffer[j+1] = (unsigned char) value;
    //PRINTF2("%d,",(int) value - PWM_CENT);
  }
  //PRINTF("\n");
  buffer[62] = 0xff; //dummy
  csum = 0;
  for ( i = 2; i < 63; i ++ ) {
    csum += buffer[i];
  }
  buffer[63] = (unsigned char)csum;
  write( sbus_fd, buffer, 64 );
  while ( time_out -- ) {
    len = read( sbus_fd, buffer, 1 );
    if ( len > 0 ) {
      break;
    }
  }
  if ( buffer[0] == SBUS_ACK ) {
  } else {
    PRINTF("SET PWM WIDTH : NG\n");
    return -1;
  }
#endif
  return 0;
}

void
sbus_setserv( int num, int value )
{
#ifdef DEBUG_SBUS
  printf("sbus_setserv(%d, %d)\n", num, value);
#endif
  if ( num < 0 || num >= 24 ) {
    PRINTF("SETSERV: INVALID NUM\n");
    return;
  }
  value *= serv_dir[num];
  value += PWM_CENT;
  serv_curr.s[num] = value;
}

int
sbus_getserv( int num )
{
  int value;

  if ( num < 0 || num >= 24 ) {
    PRINTF("GETSERV: INVALID NUM\n");
    return 0;
  }
  value = serv_curr.s[num];
  value -= PWM_CENT;
  value *= serv_dir[num];
  return value;
}

int
sbus_addserv( int num, int value )
{
#ifdef DEBUG_SBUS
  printf("sbus_addserv(%d, %d)\n", num, value);
#endif
  if ( num < 0 || num >= 24 ) {
    PRINTF("ADDSERV: INVALID NUM\n");
    return 0;
  }
  value += serv_curr.s[num];
  serv_curr.s[num] = value;
  return (value - PWM_CENT) * serv_dir[num];
}

void
sbus_printserv()
{
  int i;

  for ( i = 0; i < 24; i ++ ) {
    unsigned short value = serv_curr.s[i];
    value += use_trim * serv_trim[i];
    printf( "%d ", (int) value );
  }
  printf( "\n" );
}

void
sbus_initnext()
{
  int i;

  for ( i = 0; i < 24; i ++ ) {
    serv_next.s[i] = serv_curr.s[i];
  }
}

void
sbus_setnext( int num, int value )
{
#ifdef DEBUG_SBUS
  printf("sbus_setnext(%d, %d)\n", num, value);
#endif
  if ( num < 0 || num >= 24 ) {
    PRINTF("SETNEXT: INVALID NUM\n");
    return;
  }
  value *= serv_dir[num];
  value += PWM_CENT;
  serv_next.s[num] = value;
}

int
sbus_getnext( int num )
{
  int value;

#ifdef DEBUG_SBUS
  printf("sbus_getnext(%d)\n", num);
#endif
  if ( num < 0 || num >= 24 ) {
    PRINTF("GETNEXT: INVALID NUM\n");
    return 0;
  }
  value = serv_next.s[num];
  value -= PWM_CENT;
  value *= serv_dir[num];
  return value;
}

int
sbus_stepnext( int speed )
{
  int i;
  int notmoved = -1;

#ifdef DEBUG_SBUS
  printf("sbus_stepnext(%d)\n", speed);
#endif
  speed = speed > ASPEED_MAX ? ASPEED_MAX : speed;
  for ( i = 0; i < 24; i ++ ) {
    int delta = (int) serv_next.s[i] - (int) serv_curr.s[i];
    delta = delta > speed ? speed : delta;
    delta = delta < - speed ? - speed : delta;
    if ( delta != 0 ) {
      notmoved = 0;
      sbus_addserv( i, delta );
    }
  }
  return notmoved;
}

void
sbus_movesync( int speed )
{
  int i, j;
  int deltas[24];
  int max = 1, count;

  for ( i = 0; i < 24; i ++ ) {
    int delta = (int) serv_next.s[i] - (int) serv_curr.s[i];
    int absdelta = delta > 0 ? delta : - delta;
    deltas[i] = delta;
    max = max > absdelta ? max : absdelta;
  }
  count = (max - 1) / speed;
  if ( count > 0 ) {
    for ( i = 0; i < 24; i ++ ) {
      deltas[i] /= count + 1;
    }
    do {
      for ( i = 0; i < 24; i ++ ) {
	sbus_addserv( i, deltas[i] );
      }
      sbus_setwidth();
    } while ( -- count );
  }
  for ( i = 0; i < 24; i ++ ) {
    serv_curr.s[i] = serv_next.s[i];
  }
  sbus_setwidth();
}

void
sbus_usetrim( int value )
{
  int i;

  use_trim = value ? 1 : 0;
  for ( i = 0; i < 24; i ++ ) {
    sbus_setserv( i, 0 );
  }
}

void
sbus_printnext()
{
  int i;

  for ( i = 0; i < 24; i ++ ) {
    printf( "%d ", (int) serv_next.s[i] - PWM_CENT );
  }
  printf( "\n" );
}

unsigned short
sbus_getad(int num)
{
#ifdef DEBUG_SBUS
  printf("sbus_getad(%d)", num);
  return 0;
#else
  int len = -1;
  int time_out = SBUS_TIMEOUT1;

  if ( sbus_fd < 0 ) {
    PRINTF("SBUS OpenError\n");
    return 0;
  }
  if ( num < 0 || num > 7 ) {
    PRINTF("NG: PORT\n");
    return 0;
  }
  buffer[0] = 0x20;
  buffer[1] = (((unsigned short)(num)) << 4) | 0x01;
  //PRINTF2("ADDR = %x",buffer[0]);
  //PRINTF2("%2x\n",buffer[1]);
  write( sbus_fd, buffer, 2 );
  //PRINTF2("GET AD%d VALUE : ",num);
  while ( time_out -- ) {
    len = read( sbus_fd, buffer, BUFF_SIZE );
    if ( len > 2 ) {
      break;
    }
  }
  if ( buffer[0] == SBUS_ACK ) {
  } else {
    PRINTF("GET AD : NG\n");
    return 0;
  }
  return ((unsigned short)((int)buffer[1] << 8 | (int)buffer[2]));
#endif
}
